@@ -164,16 +164,26 @@ handle_error = function(e) {
 },
 
 send_plot = function(plotobj) {
+    log_debug('Sending plot...')
     formats <- namedlist()
     metadata <- namedlist()
     for (mime in getOption('jupyter.plot_mimetypes')) {
+        w <- attr(plotobj, '.irkernel_width')
+        h <- attr(plotobj, '.irkernel_height')
+        ppi <- attr(plotobj, '.irkernel_ppi')
         tryCatch({
-            formats[[mime]] <- mime2repr[[mime]](plotobj, attr(plotobj, '.irkernel_width'), attr(plotobj, '.irkernel_height'))
+            formats[[mime]] <- mime2repr[[mime]](plotobj, w, h)
         }, error = handle_error)
+        if (!identical(mime, 'text/plain')) {
+            metadata[[mime]] <- list(
+                width  = w * ppi,
+                height = h * ppi
+            )
+        }
         # Isolating SVGs (putting them in an iframe) avoids strange
         # interactions with CSS on the page.
         if (identical(mime, 'image/svg+xml')) {
-            metadata[[mime]] <- list(isolated = TRUE)
+            metadata[[mime]]$isolated <- TRUE
         }
     }
     publish_mimebundle(formats, metadata)
@@ -219,24 +229,26 @@ stream = function(output, streamname) {
 handle_graphics = function(plotobj) {
     log_debug('Graphics output...')
     if (!plot_builds_upon(last_recorded_plot, plotobj)) {
-        log_debug('Sending plot...')
         send_plot(last_recorded_plot)
     }
     # need to be set here to capture the size and have it available when the plot is sent
     attr(plotobj, '.irkernel_width')  <- getOption('repr.plot.width',  repr_option_defaults$repr.plot.width)
     attr(plotobj, '.irkernel_height') <- getOption('repr.plot.height', repr_option_defaults$repr.plot.height)
+    attr(plotobj, '.irkernel_ppi')    <-
+        getOption('repr.plot.res', repr_option_defaults$repr.plot.res) /
+        getOption('jupyter.plot_scale', jupyter_option_defaults$jupyter.plot_scale)
     last_recorded_plot <<- plotobj
 },
 
 handle_message = function(o) {
     log_debug('Message output: %s', o)
-    stream(paste(conditionMessage(o), collapse = ''), 'stderr')
+    stream(paste(c(conditionMessage(o), '\n'), collapse = ''), 'stderr')
 },
 
 handle_warning = function(o) {
     call <- conditionCall(o)
     call <- if (is.null(call)) '' else sprintf(' in %s', deparse(call)[[1]])
-    msg <- sprintf('Warning message%s:\n%s', call, dQuote(conditionMessage(o)))
+    msg <- sprintf('Warning message%s:\n%s\n', call, dQuote(conditionMessage(o)))
     log_debug('Warning output: %s', msg)
     stream(msg, 'stderr')
 },
@@ -299,7 +311,10 @@ execute = function(request) {
             envir = .GlobalEnv,
             output_handler = oh,
             stop_on_error = 1L),
-        interrupt = function(cond) interrupted <<- TRUE,
+        interrupt = function(cond) {
+            log_debug('Interrupt during execution')
+            interrupted <<- TRUE
+        },
         error = .self$handle_error) # evaluate does not catch errors in parsing
     
     if (!is_silent() && !is.null(last_recorded_plot)) {
@@ -310,6 +325,8 @@ execute = function(request) {
         if (interrupted) 'abort'
         else if (!is.null(err$ename)) 'error'
         else 'ok'
+
+    log_debug('Code execution result: %s', status)
     
     reply_content <- c(
         list(
